import {
  Box,
  Stack,
  TextField,
  Typography,
} from '@mui/material'
import { SubmitHandler, useForm } from 'react-hook-form'
import React, { forwardRef, Ref, useImperativeHandle } from 'react'
import { useTranslation } from 'react-i18next'
import { ContactInformationBySponsor } from './guestDataForm'
import { EnteredGuestData } from './enteredGuestData'
import { GuestRegisterCallableMethods } from './guestRegisterCallableMethods'
import { isValidMobilePhoneNumber } from '../../../utils'


interface GuestRegisterProperties {
  nextHandler(guestData: EnteredGuestData): void

  guestData: ContactInformationBySponsor
}


const GuestRegisterStep = forwardRef((props: GuestRegisterProperties, ref: Ref<GuestRegisterCallableMethods>) => {
  const { i18n, t } = useTranslation(['common'])
  const { nextHandler, guestData } = props

  const submit: SubmitHandler<EnteredGuestData> = (data) => {
    nextHandler(data)
  }

  const {
    register,
    handleSubmit,
    formState: { errors },
  } = useForm<EnteredGuestData>()
  const onSubmit = handleSubmit(submit)

  function doSubmit() {
    return onSubmit()
  }

  useImperativeHandle(ref, () => ({ doSubmit }))

  return (
    <>
      <Typography variant='h5' sx={{
        paddingTop: '1rem',
        paddingBottom: '1rem',
      }}>{t('guestRegisterWizardText.yourContactInformation')}</Typography>
      <Typography
        sx={{ paddingBottom: '2rem' }}>{t('guestRegisterWizardText.contactInformationDescription')}</Typography>
      <Box sx={{ maxWidth: '30rem' }}>
        <form onSubmit={onSubmit}>
          <Stack spacing={2}>
            <TextField
              id='firstName'
              label={t('input.firstName')}
              value={guestData.first_name}
              disabled
            />
            <TextField
              id='lastName'
              label={t('input.lastName')}
              value={guestData.last_name}
              disabled
            />

            {/* TODO Put e-mail field back */}
            {/*<TextField*/}
            {/*  id='email'*/}
            {/*  label={t('input.email')}*/}
            {/*  value={guestData.email}*/}
            {/*  disabled*/}
            {/*/>*/}

            <TextField
              id='mobilephone'
              label={t('input.mobilePhone')}
              error={!!errors.mobilePhone}
              helperText={errors.mobilePhone && errors.mobilePhone.message}
              {...register('mobilePhone', {
                validate: isValidMobilePhoneNumber,
              })}
            />

            <Typography variant='h5'
                        sx={{ paddingTop: '1rem' }}>{t('guestRegisterWizardText.yourGuestPeriod')}</Typography>
            <Typography
              sx={{ paddingBottom: '1rem' }}>{t('guestRegisterWizardText.guestPeriodDescription')}</Typography>

            <TextField id='ou-unit'
                       value={i18n.language === 'en' ? guestData.ou_name_en : guestData.ou_name_nb}
                       label={t('ou')}
                       disabled />

            <TextField
              id='roleType'
              label={t('input.roleType')}
              value={i18n.language === 'en' ? guestData.ou_name_en : guestData.ou_name_nb}
              disabled
            />

            <TextField
              id='rolePeriod'
              label={t('period')}
              value={`${guestData.role_start} - ${guestData.role_end}`}
              disabled
            />

            <TextField
              id='comment'
              label={t('input.comment')}
              multiline
              rows={5}
              value={guestData.comment}
              disabled
            />
          </Stack>
        </form>
      </Box>
    </>
  )
})

export default GuestRegisterStep
