from django.core.exceptions import ValidationError
from rest_framework import serializers

from greg.models import Person, Role, RoleType, Identity, Consent, ConsentType


class RoleSerializer(serializers.ModelSerializer):
    type = serializers.SlugRelatedField(
        queryset=RoleType.objects.all(), slug_field="identifier"
    )

    class Meta:
        model = Role
        fields = [
            "id",
            "start_date",
            "end_date",
            "sponsor_id",
            "orgunit_id",
            "created",
            "updated",
            "type",
        ]


class IdentitySerializer(serializers.ModelSerializer):
    class Meta:
        model = Identity
        fields = "__all__"

    def is_duplicate(self, identity_type: str, value: str) -> bool:
        # Guests may be verified using another unrecognised identification method,
        # which the sponsor is required to elaborate in the value column.
        # In this case we cannot assume the union of the identity type and
        # the value to be unique across all records.
        if identity_type == Identity.IdentityType.OTHER:
            return False

        # If the type is a specific ID type, then duplicates are not expected
        return Identity.objects.filter(type=identity_type).filter(value=value).exists()

    def validate(self, attrs):
        if self.is_duplicate(attrs["type"], attrs["value"]):
            raise ValidationError("Identity already exists")
        return attrs


class ConsentTypeSerializerBrief(serializers.ModelSerializer):
    class Meta:
        model = ConsentType
        fields = [
            "identifier",
            "valid_from",
            "user_allowed_to_change",
        ]


class ConsentSerializerBrief(serializers.ModelSerializer):
    type = ConsentTypeSerializerBrief(read_only=True)

    class Meta:
        model = Consent
        fields = ["type", "consent_given_at"]


class PersonSerializer(serializers.ModelSerializer):
    identities = IdentitySerializer(many=True, read_only=True)
    roles = RoleSerializer(many=True, read_only=True)
    consents = ConsentSerializerBrief(many=True, read_only=True)

    class Meta:
        model = Person
        fields = [
            "id",
            "first_name",
            "last_name",
            "date_of_birth",
            "registration_completed_date",
            "identities",
            "roles",
            "consents",
        ]
